#include "GLWidget.h"

#include <iostream>
#include <stdexcept>
#include <QtGui\qopenglcontext.h>
#include <QtGui\qvector3d.h>

#include "MainWindow.h"
#include "RENDER\global.h"


GLWidget::GLWidget(QWidget *parent)
	: QOpenGLWidget(parent)
{
	timer = new QTimer(this);
	keyPressed = FOCUS;
	setFocusPolicy(Qt::TabFocus);
	fullscreen = false;
	modelMatrix.setToIdentity();
	// NO OPENGL CALLS HERE!

}

GLWidget::~GLWidget()
{
	cleanup();
}

void GLWidget::initializeGL()
{
	makeCurrent();

	setSignalsAndSlots();
	setOpenGL();

	timer->start(16); //60FPS

	std::cout << "INIT" << std::endl;

}

void GLWidget::updateGL() {

    if (newVolume) {
        volumeRenderer->prepareNewVolume(volume);
        newVolume = false;       
    }

	camera->setMMatrix(modelMatrix);
	update(); // <-- DO NOT OVERRIDE THIS!
}

void GLWidget::paintGL()
{

    if (volume != nullptr) {
		volumeRenderer->render(*camera);
		makeCurrent();
		volumeRenderer->render(width, height, *camera);
    }

}

void GLWidget::resizeGL(int _width, int _height)
{

	width = _width;
	height = _height;
	camera->setProjection(width, height);

	glViewport(0, 0, width, height);

	std::cout << "RESIZE" << std::endl;
}

void GLWidget::setSignalsAndSlots()
{
	connect(context(), SIGNAL(aboutToBeDestroyed()), this, SLOT(cleanup()));
	connect(timer, SIGNAL(timeout()), this, SLOT(updateGL()));
}

void GLWidget::setOpenGL()
{
	glewExperimental = true;
    GLuint error = glewInit();
	if (error != GLEW_OK) {
        std::cout << glewGetErrorString(error) << std::endl;
		throw std::exception("CANNOT INIT GLEW");
	}

	volumeRenderer = new VolumeRenderer(volume);
	camera = new GLCamera(QVector3D(0, 0, 1), QVector3D(0, 0, -1));

	
	glClearColor(0, 0, 0, 1);
	glDrawBuffer(GL_BACK);

	std::cout << "OPENGL" << std::endl;

}

void GLWidget::mouseMoveEvent(QMouseEvent * event)
{
	int x = event->x() - position.x();
	int y = event->y() - position.y();
	position = event->pos();

    if (event->buttons() == Qt::LeftButton) {
        modelMatrix.rotate(-x, QVector3D(0, 1, 0));
    } else {
        modelMatrix.rotate(-y, QVector3D(1, 0, 0));
    }
}

void GLWidget::mousePressEvent(QMouseEvent* event)
{
	position = event->pos();
}

void GLWidget::mouseDoubleClickEvent(QMouseEvent* event)
{
	if (event->buttons() == Qt::LeftButton) {
		fullscreen = !fullscreen;
		std::cout << fullscreen << std::endl;

		if (fullscreen) {
			showFullScreen();
		}
		else
			showNormal();
	}
}

void GLWidget::keyPressEvent(QKeyEvent* event)
{
	keyPressed = event->key();

	switch (keyPressed)
	{
	case ZOOM:
		std::cout << "ZOOM" << std::endl;
		break;
	case BLUR:
		std::cout << "BLUR" << std::endl;
		break;
	case FOCUS:
		std::cout << "FOCUS" << std::endl;
		break;
	default:
		break;
	}

	if (keyPressed == REFRESH)
		volumeRenderer->refreshShader();
}

void GLWidget::wheelEvent(QWheelEvent* event)
{
	int wheelValue = (event->angleDelta().y() / 8) / 15;
	switch (keyPressed) {
	case ZOOM:
		camera->zoom(wheelValue);
		break;
	case BLUR:
		volumeRenderer->setBlurFactor(wheelValue);
		break;
	case FOCUS:
		volumeRenderer->setFocusPlane(wheelValue);
		break;
	}
	
}

void GLWidget::cleanup()
{
	makeCurrent();

	delete volumeRenderer;
	volumeRenderer = nullptr;

	doneCurrent();
}
