#ifndef GLWIDGET_H
#define GLWIDGET_H

#include "glew.h"
#include <QtCore\qtimer.h>
#include <QtCore\qelapsedtimer.h>
#include <QtGui\qmatrix4x4.h>
#include <QtGui\QMouseEvent>
#include <QtGui\QWheelEvent>
#include <QtGui\QKeyEvent>
#include <QtWidgets\qopenglwidget.h>

#include "..\RENDER\GLShader.h"
#include "..\RENDER\GLCamera.h"
#include "RENDER\VolumeRenderer.h"

/** This class represents a subtype of a opengl widget for rendering purposes */
class GLWidget : public QOpenGLWidget
{
	Q_OBJECT

public:
	GLWidget(QWidget *parent);
	~GLWidget();

	// AUTO-CALL, FOR SETTINGS AND CONFIGS
	void initializeGL();

	// RENDERER
	void paintGL();

	// RE-ASSIGNS VIEWPORT AND PROJECTION
	void resizeGL(int width, int height);	


private:

#define REFRESH 0x52 //r
#define ZOOM 0x5a //z
#define BLUR 0x42 //b
#define FOCUS 0x46 //f

	/** creates connections and slots */
	void setSignalsAndSlots();

	/** initializes GLEW and sets values for rendering */
	void setOpenGL();

	// MAINLY OBJECT ROTATION
	void mouseMoveEvent(QMouseEvent* event);
	void mousePressEvent(QMouseEvent* event);

	// CHANGES THE ACTUAL ZOOM/BLUR/FOCUS VALUE
	void wheelEvent(QWheelEvent* event);

	// FULLSCREEN -- NOT WORKING 
	void mouseDoubleClickEvent(QMouseEvent* event);

	// KEY INPUT FOR ZOOM/BLUR/FOCUS/REFRESH
	void keyPressEvent(QKeyEvent* event);

	/** the volume rendering class */
    VolumeRenderer* volumeRenderer = nullptr;

	/** timer for frame and update refresh purposes */
	QTimer* timer = nullptr;
	
	/** the virtual camera in the scene */
	GLCamera* camera = nullptr;

	/** fullscreen mode activation -- not used */
	bool fullscreen;

	/** flag to start rendering process */
    bool volumeLoaded;

	/** stores the numeric value of pressed key */
	int keyPressed;

	/** width of the widget */
	GLint width;

	/** height of the widget */
	GLint height;

	/** stores the actual mouse position */
	QPoint position;
	
	/** model matrix for scene transformation purposes */
	QMatrix4x4 modelMatrix;

	private slots:

	/** updates the camera, the widget and checks if a new volume has been loaded */
	void updateGL();

	/** deletes the volume renderer */
	void cleanup();
};

#endif // GLWIDGET_H
