#include "MainWindow.h"
#include "RENDER\global.h"

#include <QtWidgets\qfiledialog.h>
#include <QtWidgets\qstylepainter.h>
#include <QtWidgets\qmessagebox.h>




MainWindow::MainWindow(QWidget *parent)
	: QMainWindow(parent), m_Volume(0)
{
	m_Ui = new Ui_MainWindow();
	m_Ui->setupUi(this);

	connect(m_Ui->actionOpen, SIGNAL(triggered()), this, SLOT(openFileAction()));
	connect(m_Ui->actionClose, SIGNAL(triggered()), this, SLOT(closeAction()));	
	connect(m_Ui->actionSettings, SIGNAL(triggered()), this, SLOT(openSettingDialog()));
	connect(m_Ui->actionNew, SIGNAL(triggered()), this, SLOT(newTransferDialog()));
	connect(m_Ui->action1, SIGNAL(triggered()), this, SLOT(editTransfer1Dialog()));
	connect(m_Ui->action2, SIGNAL(triggered()), this, SLOT(editTransfer2Dialog()));
	connect(m_Ui->action3, SIGNAL(triggered()), this, SLOT(editTransfer3Dialog()));
	connect(m_Ui->action4, SIGNAL(triggered()), this, SLOT(editTransfer4Dialog()));
	connect(m_Ui->action5, SIGNAL(triggered()), this, SLOT(editTransfer5Dialog()));
}

MainWindow::~MainWindow()
{
	delete m_Volume;

	for (int i = 0; i < tfsSize; i++) {
		delete tfs[i];
		tfs[i] = nullptr;
	}
}


//-------------------------------------------------------------------------------------------------
// Slots
//-------------------------------------------------------------------------------------------------

void MainWindow::openFileAction()
{
	QString filename = QFileDialog::getOpenFileName(this, "Data File", 0, tr("Data Files (*.dat)"));

	if (!filename.isEmpty())
	{
		// store filename
		m_FileType.filename = filename;
		std::string fn = filename.toStdString();
		bool success = false;

		// progress bar and top label
		m_Ui->progressBar->setEnabled(true);

		setWindowTitle("Loading data ...");

		// load data according to file extension
		if (fn.substr(fn.find_last_of(".") + 1) == "dat")		// LOAD VOLUME
		{
			// create VOLUME
			m_FileType.type = VOLUME;

            // cleanup before new volume
            if (m_Volume != nullptr) {
                delete m_Volume;
            }

			m_Volume = new Volume();

			// load file
			success = m_Volume->loadFromFile(filename, m_Ui->progressBar);
		}

		m_Ui->progressBar->setEnabled(false);

		// status message
		if (success)
		{
			setWindowTitle(filename);
		}
		else
		{
			setWindowTitle("ERROR loading file!");
			m_Ui->progressBar->setValue(0);
		}
	}

    volume = m_Volume;
    newVolume = true;
}

void MainWindow::closeAction()
{
	close();
}

void MainWindow::openSettingDialog()
{
	view = new ViewWindow(this);
	view->show();
}

void MainWindow::newTransferDialog()
{
	openTransferDialog(-1, tfsSize == 5, ERR_SIZE);
}

void MainWindow::editTransfer1Dialog()
{
	int index = m_Ui->action1->text().toInt();
	openTransferDialog(index, tfsSize < index, ERR_INDEX);
}

void MainWindow::editTransfer2Dialog()
{
	int index = m_Ui->action2->text().toInt();
	openTransferDialog(index, tfsSize < index, ERR_INDEX);
}

void MainWindow::editTransfer3Dialog()
{
	int index = m_Ui->action3->text().toInt();
	openTransferDialog(index, tfsSize < index, ERR_INDEX);
}

void MainWindow::editTransfer4Dialog()
{
	int index = m_Ui->action4->text().toInt();
	openTransferDialog(index, tfsSize < index, ERR_INDEX);
}

void MainWindow::editTransfer5Dialog()
{
	int index = m_Ui->action5->text().toInt();
	openTransferDialog(index, tfsSize < index, ERR_INDEX);
}

void MainWindow::openTransferDialog(int ind, bool check, const QString& message)
{
	if (check)
		QMessageBox::warning(this, "WARNING", message);
	else {
		transfer = new TransferWindow(this, ind);
		transfer->show();
	}
}
