#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "../GeneratedFiles/ui_MainWindow.h"
#include "Volume.h"

#include <QtWidgets\qwidget.h>
#include <QtWidgets\qlabel.h>
#include <QtWidgets\qprogressbar.h>
#include <QtWidgets\qstatusbar.h>
#include <QtWidgets\qpushbutton.h>
#include <QtWidgets\qmainwindow.h>

//#include <QVariant>
#include "ViewWindow.h"
#include "TransferWindow.h"

/** A class representing the main window containing all user interfaces and rendering window */
class MainWindow : public QMainWindow
{

	Q_OBJECT

public:

	MainWindow(QWidget *parent = 0);
	~MainWindow();


	protected slots :

	/** reads the volume information from a selected file when the menu-option OPEN is clicked */
	void openFileAction();

	/** closes and stops this program */
	void closeAction();

	/** opens the settings dialog to update the focus plane and blurring value */
	void openSettingDialog();

	/** creates a new transfer function, up to two new functions can be created  */
	void newTransferDialog();

	/** edits the first pre-set transfer function */
	void editTransfer1Dialog();

	/** edits the second pre-set transfer function */
	void editTransfer2Dialog();

	/** edits the third pre-set transfer function */
	void editTransfer3Dialog();

	/** edits the fourth transfer function if it was previously created */
	void editTransfer4Dialog();

	/** edits the fifth transfer function if it was previously created */
	void editTransfer5Dialog();
	


private:

	// USER INTERFACE ELEMENTS

	/** the object for user interactions */
	Ui_MainWindow *m_Ui;

	/** the rendering window */
    GLWidget *glWidget;

	/** the settings dialog window */
	ViewWindow* view;

	/** the transfer function dialog window */
	TransferWindow* transfer;

	/** error message if an unset transfer function is chosen for editing */
	const QString ERR_INDEX = "Unset transfer function!";

	/** error message if more than two new transfer functions are created */
	const QString ERR_SIZE = "Max number of transfer functions reached!";
	// DATA 

	/** opens a transfer window dialog if a certain condition is met, else it throws an error message 
	* @params ind the index of the transfer function to be set
	* @params check if true: open the dialog with index ind, else throw error message
	* @params message the error message
	*/
	void openTransferDialog(int ind, bool check, const QString& message);

	enum DataType
	{
		VOLUME = 0
	};

	struct FileType
	{
		QString	filename;
		DataType type;
	} m_FileType;

	/** the volume to be rendered */
	Volume *m_Volume;

};

#endif
