#include "TransferWindow.h"

TransferWindow::TransferWindow(QWidget* parent, int editIndex) :
	QDialog(parent)
{
	setAttribute(Qt::WA_DeleteOnClose);
	ui = new Ui_Transfer();
	ui->setupUi(this);
	QWidget::setFixedSize(size());

	// new
	if (editIndex < 0) {
		index = tfsSize++;
		tfs[index] = new TransferFunction();
		newFunction = true;
	}
	// edit
	else {
		
		index = editIndex - 1;
		newFunction = false;
	}
		

	baseFirstValue = tfs[index]->firstValue;
	baseSecondValue = tfs[index]->secondValue;
	baseOptValue = tfs[index]->optValue;
	baseColorValue[0] = tfs[index]->color[0];
	baseColorValue[1] = tfs[index]->color[1];
	baseColorValue[2] = tfs[index]->color[2];

	ui->firstSlider->setValue(baseFirstValue);
	ui->secondSlider->setValue(baseSecondValue);
	ui->thirdSlider->setValue(baseOptValue * 100);
	ui->firstValue->setText(QString::number(baseFirstValue));
	ui->secondValue->setText(QString::number(baseSecondValue));
	ui->thirdValue->setText(QString::number(baseOptValue * 100));

	connect(ui->firstSlider, SIGNAL(valueChanged(int)), this, SLOT(setTransferFunction()));
	connect(ui->secondSlider, SIGNAL(valueChanged(int)), this, SLOT(setTransferFunction()));
	connect(ui->thirdSlider, SIGNAL(valueChanged(int)), this, SLOT(setTransferFunction()));

	connect(ui->colorButton, SIGNAL(clicked()), this, SLOT(getColor()));
	connect(ui->buttonBox, SIGNAL(accepted()), this, SLOT(close()));
	connect(ui->buttonBox, SIGNAL(rejected()), this, SLOT(rejectChanges()));
	
}


TransferWindow::~TransferWindow()
{
}

void TransferWindow::setTransferFunction()
{
	int val1 = ui->firstSlider->value();
	int val2 = ui->secondSlider->value();
	int val3 = ui->thirdSlider->value();

	if (val1 > val2) {
		val1 = val2;
		ui->firstSlider->setValue(val1);
	}

	ui->firstValue->setText(QString::number(val1));
	ui->secondValue->setText(QString::number(val2));
	ui->thirdValue->setText(QString::number(val3));

	tfs[index]->firstValue = val1;
	tfs[index]->secondValue = val2;
	tfs[index]->optValue = val3 / 100.f;
}

void TransferWindow::rejectChanges()
{
	if (newFunction == true) {
		delete tfs[index];
		tfs[index] = nullptr;
		tfsSize--;
		newFunction = false;
	}
	else {
		tfs[index]->firstValue = baseFirstValue;
		tfs[index]->secondValue = baseSecondValue;
		tfs[index]->optValue = baseOptValue;
		tfs[index]->color[0] = baseColorValue[0];
		tfs[index]->color[1] = baseColorValue[1];
		tfs[index]->color[2] = baseColorValue[2];
	}


	close();
}

void TransferWindow::getColor() {
	QColor color = QColorDialog::getColor(Qt::blue, this);
	QString qss = QString("background-color: %1").arg(color.name());

	ui->colorButton->setStyleSheet(qss);
	tfs[index]->color[0] = color.redF();
	tfs[index]->color[1] = color.greenF();
	tfs[index]->color[2] = color.blueF();
}
