#include "ViewWindow.h"
#include "RENDER\global.h"
#include <QtCore\qmath.h>

ViewWindow::ViewWindow(QWidget *parent):
QDialog(parent)
{
	setAttribute(Qt::WA_DeleteOnClose);

	baseFocusValue = focusPlane;
	baseBlurValue = blurAccum;

	ui = new Ui_Dialog();
	ui->setupUi(this);
	QWidget::setFixedSize(size());

	ui->focusSlider->setRange(0, lastFocusPlane);
	ui->focusSlider->setValue(baseFocusValue);
	ui->focusValue->setText(QString::number(baseFocusValue + 1));

	ui->blurSlider->setRange(0, (int)maxBlurAttenuation);
	ui->blurSlider->setValue(baseBlurValue);
	ui->blurValue->setText(QString::number(toPercentage(baseBlurValue, maxBlurAttenuation)));
	
	connect(ui->focusSlider, SIGNAL(valueChanged(int)), this, SLOT(getFocusSliderValue()));
	connect(ui->blurSlider, SIGNAL(valueChanged(int)), this, SLOT(getBlurSliderValue()));
	connect(ui->buttonBox, SIGNAL(accepted()), this, SLOT(close()));
	connect(ui->buttonBox, SIGNAL(rejected()), this, SLOT(rejectValue()));
}


ViewWindow::~ViewWindow()
{
}

void ViewWindow::getBlurSliderValue()
{
	blurAccum = ui->blurSlider->value();
	blurAttenuation = qTan(qDegreesToRadians(blurAccum));
	ui->blurValue->setText(QString::number(toPercentage(blurAccum, maxBlurAttenuation)));
}

void ViewWindow::getFocusSliderValue() {
	focusPlane = ui->focusSlider->value();
	ui->focusValue->setText(QString::number(focusPlane + 1));
}

void ViewWindow::rejectValue()
{
	focusPlane = baseFocusValue;
	blurAccum = baseBlurValue;
	blurAttenuation = qTan(qDegreesToRadians(blurAccum));
	close();
}


int ViewWindow::toPercentage(const float & value, const float & max)
{
	return (int)((value / max) * 100);
}