#pragma once

#include <QtGui\qvector3d.h>
#include <QtGui\qvector2d.h>
#include <QtGui\qmatrix4x4.h>
#include <QtCore\qmath.h>

/** A class for representing a virtual camera in the scene */
class GLCamera
{
public:
	GLCamera(const QVector3D& _pos, const QVector3D& _target);
	~GLCamera();

    /** sets the perspectiv projection matrix 
	* @param width the widget's width
	* @param height the widget's height
	*/
	void setProjection(const int& width, const int& height);

    /** zooms into/out of the screen */
	void zoom(const int& faktor);

    /** sets the model matrix
	* @param modelMatrix adjusted model matrix
	*/
	void setMMatrix(const QMatrix4x4& modelMatrix);

	/** returns the projection view model matrix
	* @return 4x4 projection view model matrix
	*/
	const QMatrix4x4 getPVMMatrix() const;

	/** returns the projection matrix 
	* @return 4x4 projection matrix
	*/
	const QMatrix4x4 getProjection() const;

	/** returns the view matrix 
	* @return 4x4 view matrix
	*/
    const QMatrix4x4 getVMatrix() const;

	/** returns the model matrix 
	* @return 4x4 model matrix
	*/
    const QMatrix4x4 getModelMatrix() const;

private:

    /** camera position */
	QVector3D position;

    /** camera look at */
	QVector3D target;

	/** camera up */
	const QVector3D& UP = QVector3D(0, 1, 0);

	/** the view matrix */
	QMatrix4x4 viewMatrix;

	/** the perspective projection matrix */
	QMatrix4x4 projMatrix;

	/** the view model matrix */
	QMatrix4x4 projViewModelMatrix;

	/** the model matrix */
    QMatrix4x4 modelMatrix;

};

