#pragma once

#include "glew.h"
#include <string>
#include <QtGui\qopenglshaderprogram.h>

#include "GLCamera.h"

/** General class for shaders */
class GLShader
{
public:
	GLShader(const QString& _vertex, const QString& _fragment);
	~GLShader();

    /** use program and bind the vao
     * @param vao: binds this vao
     */
    void prepare(const GLuint& vao);

    /** unbind everything */
	void cleanUp();

    /** returns if the program is usable
     * @return true, if no error occured during compiling and linking, false else
     */
	const bool isRenderable() const;
	
protected:
	/** gets all necessary uniform locations */
	virtual void configShader() = 0;

    /** Qt object for creating the shader */
	QOpenGLShaderProgram* linker;

    /** shader program ID */
	GLuint program;

	/** creates, compiles and links the shader
     * @param _vertex: path to the vertex shader
     * @param _fragment: path to the fragment shader
     */
	void createShader(const QString& _vertex, const QString& _fragment);

    /** bool holding the info if the shader can be used */
	GLboolean renderable;
};

