#include "ProxyGeometry.h"

ProxyGeometry::ProxyGeometry(QVector3D cubeSize) {
    cube = new PolygonIntersection::Cube(cubeSize);
}

ProxyGeometry::~ProxyGeometry() {
    delete cube;
}

void ProxyGeometry::updateProxyGeometry(QMatrix4x4 modelMatrix, int samples) {

    planeVertices.clear();
    planeVertices.reserve(samples);
    texCoords.clear();
    texCoords.reserve(samples);

    QMatrix4x4 invModMatrix = modelMatrix.inverted();
    cube->transform(modelMatrix);
    float maxZ = -FLT_MAX;
    float minZ = FLT_MAX;
    int index = -1;

    for (unsigned int i = 0; i < cube->points.size(); i++) {
        float tmp = cube->points[i].z();
        maxZ = qMax(tmp, maxZ);
        if (tmp < minZ) {
            minZ = tmp;
            index = i;
        }
    }

    sliceDistance = qAbs((maxZ - minZ)) / samples;

    QVector3D normal = QVector3D(0, 0, -1);
    QVector4D plane = QVector4D(normal, - QVector3D::dotProduct(normal, QVector3D(0, 0, 1)));

    for (unsigned int i = 0; i < samples; i++) {
        plane.setW(- (QVector3D::dotProduct(normal, QVector3D(0, 0, maxZ - i*sliceDistance))));
        planeVertices.push_back(PolygonIntersection::calcPolygon(plane, *cube, index));
        std::vector<QVector3D> texCoords;
        texCoords.reserve(planeVertices[i].size());
        for (unsigned int j = 0; j < planeVertices[i].size(); j++) {
            QVector3D point = ((invModMatrix * planeVertices[i][j]) + (cube->sizes / 2.0f));
            point = point / cube->sizes;
            texCoords.push_back(point);
        }
        
        this->texCoords.push_back(texCoords);
    }

}

std::vector < std::vector<QVector3D>>& ProxyGeometry::getPlaneVertices() {
    return planeVertices;
}

std::vector < std::vector<QVector3D>>& ProxyGeometry::getTexCoords() {
    return texCoords;
}

float ProxyGeometry::getSliceDistance() {
    return sliceDistance;
}

std::vector<QVector3D> PolygonIntersection::calcPolygon(QVector4D normal, Cube& cube, int index) {
    std::vector<QVector3D> vertices;
    vertices.reserve(6);
    for (unsigned int i = 0; i < cube.edgeOrder[index].size(); i++) {
        int edgeIndex = cube.edgeOrder[index][i];
        if (cross(*cube.edges[edgeIndex].a, *cube.edges[edgeIndex].b, normal)) {
            vertices.push_back(intersect(*cube.edges[edgeIndex].a, *cube.edges[edgeIndex].b, normal));
        }
    }

    return vertices;
}

bool PolygonIntersection::inside(QVector3D vertex, QVector4D normal) {
    return QVector4D::dotProduct(normal, QVector4D(vertex, 1)) > 0 ? false : true;
}

bool PolygonIntersection::cross(QVector3D a, QVector3D b, QVector4D normal) {
    if ((inside(a, normal) && !inside(b, normal)) || (!inside(a, normal) && inside(b, normal)))
        return true;

    return false;
}

QVector3D PolygonIntersection::intersect(QVector3D a, QVector3D b, QVector4D normal) {
    float la = QVector4D::dotProduct(QVector4D(a, 1), normal);
    float lb = QVector4D::dotProduct(QVector4D(b, 1), normal);
    float t = la / (la - lb);

  /*  if (inside(a, normal))
        t -= 0.000001f;
    if (inside(b, normal))
        t += 0.000001f;*/

    return interpolate(a, b, t);
   /* QVector3D direction = b - a;
    QVector3D planePoint = QVector3D(0, 0, normal.w());
    QVector3D planeNormal = QVector3D(normal);
    float s = QVector3D::dotProduct(planeNormal, planePoint - a) / (QVector3D::dotProduct(planeNormal, direction));

    return a + s*direction;*/
}

QVector3D PolygonIntersection::interpolate(QVector3D a, QVector3D b, float t) {
    return QVector3D(a * (1.0f - t) + b * t);
}
