#include "DoFShader.h"

#include <QtCore\qmath.h>

DoFShader::DoFShader() :
	GLShader("SHADER/DoFShaderVertex", "SHADER/DoFShaderFragment")
{
	configShader();
}

void DoFShader::configShader()
{

	if (renderable) {
		// VERTEX SHADER
		depthCoord = glGetUniformLocation(program, "depthCoord");
		distance = glGetUniformLocation(program, "distance");
		circleOfConfLocation = glGetUniformLocation(program, "circleOfConf");

		// FRAGMENT SHADER
		volumeTextureLocation = glGetUniformLocation(program, "volumeTexture");
		previousTextureLocation = glGetUniformLocation(program, "previousTexture");
		FRONT2BACK_Location = glGetUniformLocation(program, "FRONT2BACK");

		QString format = QString("tf[%1].%2");
		for (int i = 0; i < 5; i++) {
			std::string input = format.arg(QString::number(i),"firstValue").toStdString();
			tfFirstValueLocation[i] = glGetUniformLocation(program, input.c_str());

			input = format.arg(QString::number(i), "secondValue").toStdString();
			tfSecondValueLocation[i] = glGetUniformLocation(program, input.c_str());

			input = format.arg(QString::number(i), "color").toStdString();
			tfColorLocation[i] = glGetUniformLocation(program, input.c_str());

			input = format.arg(QString::number(i), "optValue").toStdString();
			tfOptValueLocation[i] = glGetUniformLocation(program, input.c_str());
		}

		numberOfTFLocation = glGetUniformLocation(program, "numberOfTF");
	}

}

DoFShader::~DoFShader()
{
}

void DoFShader::prepare(GLCamera& camera, const GLuint& quadVAO, const GLuint& volTEX)
{
	GLShader::prepare(quadVAO);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, volTEX);
	glUniform1i(volumeTextureLocation, 0);

	//TEST
	for (int i = 0; i < tfsSize; i++) {
		glUniform1i(tfFirstValueLocation[i], tfs[i]->firstValue);
		glUniform1i(tfSecondValueLocation[i], tfs[i]->secondValue);
		glUniform1f(tfOptValueLocation[i], tfs[i]->optValue);
		glUniform3f(tfColorLocation[i], tfs[i]->color[0], tfs[i]->color[1], tfs[i]->color[2]);
	}
	
	glUniform1i(numberOfTFLocation, tfsSize);
}

void DoFShader::draw(FBO* prev, FBO* next, float C, bool f2b)
{
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, prev->getTXO());
	glUniform1i(previousTextureLocation, 1);

	glBindFramebuffer(GL_FRAMEBUFFER, next->getID());
	glDisable(GL_BLEND);

	glUniform1f(circleOfConfLocation, C);
	glUniform1i(FRONT2BACK_Location, f2b);

	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	glBindTexture(GL_TEXTURE_2D, 0);
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
}


void DoFShader::loadDepth(float depthZCoord, float distance)
{
	glUniform1f(this->depthCoord, depthZCoord);
	glUniform1f(this->distance, distance);
}

void DoFShader::cleanUp()
{
	//glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_3D, 0);
	glBindTexture(GL_TEXTURE_2D, 0);
	GLShader::cleanUp();
}

void DoFShader::refresh()
{
	delete linker;
	createShader("SHADER/DoFShaderVertex", "SHADER/DoFShaderFragment");
	configShader();
	std::cout << "REFRESH" << std::endl;
}