#pragma once

#include <iostream>

#include "RENDER\global.h"
#include "RENDER\GLShader.h"
#include "RENDER\FBO.h"

/**
 * A class for rendering the DoF effect
 */
class DoFShader: public GLShader
{
public:
	DoFShader();
	~DoFShader();

    /** 
     * renders a slice of the volume
     * @param fbo0: previous framebuffer
     * @param fbo1: current framebuffer
     * @param C: circle of confusion
     * @param f2b: bool indicating if rendered front to back or back to front
     */
	void draw(FBO* fbo0, FBO* fbo1, float C, bool f2b);

    /** 
     * loads up the depth values to the shader
     * @param depthZCoord: distance from camera to slice
     * @param distance: distance between slices
     */
	void loadDepth(float depthZCoord, float distance);

    /** bind matrices, VAO and texture
     * @param camera: contains model, view, projection matrices
     * @param quadVAO: VAO for a simple quad
     * @param volTEX: ID for the current volume slice
     */
	void prepare(GLCamera& camera, const GLuint& quadVAO, const GLuint& volTEX);

    /** unbind everything */
	void cleanUp();

    /** recompile the shader, for debugging */
	void refresh();

private:
	void configShader() override;
	
    /** uniform location 
     * indicates the depth between camera and the current slice
     */
	GLint depthCoord;

    /** uniform location
     * indicates the distance between two slices
     */
    GLint distance;

    /** uniform location
     * indicates if the slices are rendered F2B or B2F 
     * 1: F2B, otherwise: B2F
     */
    GLint FRONT2BACK_Location;

    /** uniform location
     * circle of confusion
     */
	GLint circleOfConfLocation;

    /** uniform location
     * location of volume texture
     */
	GLint volumeTextureLocation;

    /** uniform location
     * location of previous slice texture
     */
	GLint previousTextureLocation;

	/** uniform location
	* location of the min value in the transfer function structur
	*/
	GLint tfFirstValueLocation[5];
	
	/** uniform location
	* location of the max value in the transfer function structur
	*/
	GLint tfSecondValueLocation[5];
	
	/** uniform location
	* location of the opacity value in the transfer function structur
	*/
	GLint tfOptValueLocation[5];
	
	/** uniform location
	* location of the color value in the transfer function structur
	*/
	GLint tfColorLocation[5];
	
	/** uniform location
	* location of the number of set transfer functions
	*/
	GLint numberOfTFLocation;
};

